from DobotRPC import DobotlinkAdapter, RPCClient


class MagicianApi(object):
    def __init__(self):
        self.__port_name: str
        self.__dobotlink = DobotlinkAdapter(RPCClient(), is_sync=True)

    def set_portname(self, port_name):
        self.__port_name = port_name

    def search_dobot(self):
        return self.__dobotlink.Magician.SearchDobot()

    def connect_dobot(self, queue_start=True):
        return self.__dobotlink.Magician.ConnectDobot(
            portName=self.__port_name, queueStart=queue_start)

    def disconnect_dobot(self, queue_stop=True, queue_clear=True):
        return self.__dobotlink.Magician.DisconnectDobot(
            portName=self.__port_name,
            queueStop=queue_stop,
            queueClear=queue_clear)

    def get_devicesn(self):
        return self.__dobotlink.Magician.GetDeviceSN(portName=self.__port_name)

    def set_devicename(self, device_name):
        return self.__dobotlink.Magician.SetDeviceName(
            portName=self.__port_name, deviceName=device_name)

    def get_devicename(self):
        return self.__dobotlink.Magician.GetDeviceName(
            portName=self.__port_name)

    def get_deviceversion(self):
        return self.__dobotlink.Magician.GetDeviceVersion(
            portName=self.__port_name)

    def set_devicewithl(self, enable=True, version=1):
        return self.__dobotlink.Magician.SetDeviceWithL(
            portName=self.__port_name, enable=enable, version=version)

    def get_devicewithl(self):
        return self.__dobotlink.Magician.GetDeviceWithL(
            portName=self.__port_name)

    def get_devicetime(self):
        return self.__dobotlink.Magician.GetDeviceTime(
            portName=self.__port_name)

    def get_deviceid(self):
        return self.__dobotlink.Magician.GetDeviceID(portName=self.__port_name)

    def get_productname(self):
        return self.__dobotlink.Magician.GetProductName(
            portName=self.__port_name)

    def queuedcmd_start(self):
        return self.__dobotlink.Magician.QueuedCmdStart(
            portName=self.__port_name)

    def queuedcmd_stop(self, force_stop=False):
        return self.__dobotlink.Magician.QueuedCmdStop(
            portName=self.__port_name, forceStop=force_stop)

    def queuedcmd_clear(self):
        return self.__dobotlink.Magician.QueuedCmdClear(
            portName=self.__port_name)

    def queuedcmd_startdownload(self, total_loop, lineper_loop):
        return self.__dobotlink.Magician.QueuedCmdStartDownload(
            portName=self.__port_name,
            totalLoop=total_loop,
            linePerLoop=lineper_loop)

    def queuedcmd_stopdownload(self):
        return self.__dobotlink.Magician.QueuedCmdStopDownload(
            portName=self.__port_name)

    def get_queuedcmd_currentindex(self):
        return self.__dobotlink.Magician.GetQueuedCmdCurrentIndex(
            portName=self.__port_name)

    def get_queuedcmd_leftspace(self):
        return self.__dobotlink.Magician.GetQueuedCmdLeftSpace(
            portName=self.__port_name)

    def set_armspeed_ratio(self,
                           set_type: int,
                           set_value: int,
                           is_queued=False):
        return self.__dobotlink.Magician.SetArmSpeedRatio(
            portName=self.__port_name,
            type=set_type,
            value=set_value,
            isQueued=is_queued)

    def get_armspeed_ratio(self, get_type: int):
        return self.__dobotlink.Magician.GetArmSpeedRatio(
            portName=self.__port_name, type=get_type)

    def set_servoangle(self, index: int, set_value: float, is_queued=False):
        return self.__dobotlink.Magician.SetServoAngle(
            portName=self.__port_name,
            index=index,
            value=set_value,
            isQueued=is_queued)

    def get_servoangle(self, index: int):
        return self.__dobotlink.Magician.GetServoAngle(
            portName=self.__port_name, index=index)

    def set_lspeed_ratio(self, set_type: int, set_value: int, is_queued=False):
        return self.__dobotlink.Magician.SetLSpeedRatio(
            portName=self.__port_name,
            type=set_type,
            value=set_value,
            isQueued=is_queued)

    def get_lspeed_ratio(self, get_type: int):
        return self.__dobotlink.Magician.GetLSpeedRatio(
            portName=self.__port_name, type=get_type)

    def get_pose(self):
        return self.__dobotlink.Magician.GetPose(portName=self.__port_name)

    def reset_pose(self,
                   manual_enable,
                   rear_armangle=None,
                   front_armangle=None):
        return self.__dobotlink.Magician.ResetPose(
            portName=self.__port_name,
            manualEnable=manual_enable,
            rearArmAngle=rear_armangle,
            frontArmAngle=front_armangle)

    def get_posel(self):
        return self.__dobotlink.Magician.GetPoseL(portName=self.__port_name)

    def get_alarms_state(self):
        return self.__dobotlink.Magician.GetAlarmsState(
            portName=self.__port_name)

    def clear_allalarms_state(self):
        return self.__dobotlink.Magician.ClearAllAlarmsState(
            portName=self.__port_name)

    def set_homeparams(self,
                       x: float,
                       y: float,
                       z: float,
                       r: float,
                       is_queued=False):
        return self.__dobotlink.Magician.SetHOMEParams(
            portName=self.__port_name, x=x, y=y, z=z, r=r, isQueued=is_queued)

    def get_homeparams(self):
        return self.__dobotlink.Magician.GetHOMEParams(
            portName=self.__port_name)

    def set_homecmd(self,
                    is_queued=True,
                    iswait_forfinish=True,
                    time_out=25000):
        return self.__dobotlink.Magician.SetHOMECmd(
            portName=self.__port_name,
            isQueued=is_queued,
            isWaitForFinish=iswait_forfinish,
            timeout=time_out)

    def set_autoleveling(self,
                         enable: bool,
                         precision: float,
                         is_queued=False):
        return self.__dobotlink.Magician.SetAutoLeveling(
            portName=self.__port_name,
            enable=enable,
            precision=precision,
            isQueued=is_queued)

    def get_autoleveling(self):
        return self.__dobotlink.Magician.GetAutoLeveling(
            portName=self.__port_name)

    def set_hhttrig_mode(self, mode: int, is_queued=False):
        return self.__dobotlink.Magician.SetHHTTrigMode(
            portName=self.__port_name, mode=mode, isQueued=is_queued)

    def get_hhttrig_mode(self):
        return self.__dobotlink.Magician.GetHHTTrigMode(
            portName=self.__port_name)

    def set_hhttrig_output_enabled(self, enable: bool, is_queued=False):
        return self.__dobotlink.Magician.SetHHTTrigOutputEnabled(
            portName=self.__port_name, enable=enable, isQueued=is_queued)

    def get_hhttrig_output_enabled(self):
        return self.__dobotlink.Magician.GetHHTTrigOutputEnabled(
            portName=self.__port_name)

    def get_hhttrig_output(self):
        return self.__dobotlink.Magician.GetHHTTrigOutput(
            portName=self.__port_name)

    def set_endeffector_params(self,
                               x_offset: float,
                               y_offset: float,
                               z_offset: float,
                               is_queued=False):
        return self.__dobotlink.Magician.SetEndEffectorParams(
            portName=self.__port_name,
            xOffset=x_offset,
            yOffset=y_offset,
            zOffset=z_offset,
            isQueued=is_queued)

    def get_endeffector_params(self):
        return self.__dobotlink.Magician.GetEndEffectorParams(
            portName=self.__port_name)

    def set_endeffector_type(self, set_type: int, is_queued=False):
        return self.__dobotlink.Magician.SetEndEffectorType(
            portName=self.__port_name, type=set_type, isQueued=is_queued)

    def get_endeffector_type(self):
        return self.__dobotlink.Magician.GetEndEffectorType(
            portName=self.__port_name)

    def set_endeffector_laser(self, enable: bool, on: bool, is_queued=False):
        return self.__dobotlink.Magician.SetEndEffectorLaser(
            portName=self.__port_name,
            enable=enable,
            on=on,
            isQueued=is_queued)

    def get_endeffector_laser(self):
        return self.__dobotlink.Magician.GetEndEffectorLaser(
            portName=self.__port_name)

    def set_endeffector_suctioncup(self,
                                   enable: bool,
                                   on: bool,
                                   is_queued=False):
        return self.__dobotlink.Magician.SetEndEffectorSuctionCup(
            portName=self.__port_name,
            enable=enable,
            on=on,
            isQueued=is_queued)

    def get_endeffector_suctioncup(self):
        return self.__dobotlink.Magician.GetEndEffectorSuctionCup(
            portName=self.__port_name)

    def set_endeffector_gripper(self, enable: bool, on: bool, is_queued=False):
        return self.__dobotlink.Magician.SetEndEffectorGripper(
            portName=self.__port_name,
            enable=enable,
            on=on,
            isQueued=is_queued)

    def get_endeffector_gripper(self):
        return self.__dobotlink.Magician.GetEndEffectorGripper(
            portName=self.__port_name)

    def set_jogjoint_params(self, velocity, acceleration, is_queued=False):
        return self.__dobotlink.Magician.SetJOGJointParams(
            portName=self.__port_name,
            velocity=velocity,
            acceleration=acceleration,
            isQueued=is_queued)

    def get_jogjoint_params(self):
        return self.__dobotlink.Magician.GetJOGJointParams(
            portName=self.__port_name)

    def set_jogcoordinate_params(self,
                                 velocity,
                                 acceleration,
                                 is_queued=False):
        return self.__dobotlink.Magician.SetJOGCoordinateParams(
            portName=self.__port_name,
            velocity=velocity,
            acceleration=acceleration,
            isQueued=is_queued)

    def get_jogcoordinate_params(self):
        return self.__dobotlink.Magician.GetJOGCoordinateParams(
            portName=self.__port_name)

    def set_jogcommon_params(self,
                             velocity_ratio,
                             acceleration_ratio,
                             is_queued=False):
        return self.__dobotlink.Magician.SetJOGCommonParams(
            portName=self.__port_name,
            velocityRatio=velocity_ratio,
            accelerationRatio=acceleration_ratio,
            isQueued=is_queued)

    def get_jogcommon_params(self):
        return self.__dobotlink.Magician.GetJOGCommonParams(
            portName=self.__port_name)

    def set_jogl_params(self, velocity, acceleration, is_queued=False):
        return self.__dobotlink.Magician.SetJOGLParams(
            portName=self.__port_name,
            velocity=velocity,
            acceleration=acceleration,
            isQueued=is_queued)

    def get_jogl_params(self):
        return self.__dobotlink.Magician.GetJOGLParams(
            portName=self.__port_name)

    def set_jogcmd(self, is_joint, cmd, is_queued):
        return self.__dobotlink.Magician.SetJOGCmd(portName=self.__port_name,
                                                   isJoint=is_joint,
                                                   cmd=cmd,
                                                   isQueued=is_queued)

    def set_ptpcmd(self,
                   ptp_mode,
                   x,
                   y,
                   z,
                   r,
                   is_queued=True,
                   iswait_forfinish=True):
        return self.__dobotlink.Magician.SetPTPCmd(
            portName=self.__port_name,
            ptpMode=ptp_mode,
            x=x,
            y=y,
            z=z,
            r=r,
            isQueued=is_queued,
            isWaitForFinish=iswait_forfinish)

    def set_rcmd(self,
                 r: int,
                 is_queued=True,
                 iswait_forfinish=True,
                 time_out=5000):
        return self.__dobotlink.Magician.SetRCmd(
            portName=self.__port_name,
            r=r,
            isQueued=is_queued,
            isWaitForFinish=iswait_forfinish,
            timeout=time_out)

    def set_ptpwithl_cmd(self,
                         ptp_mode: int,
                         x: float,
                         y: float,
                         z: float,
                         r: float,
                         l: float,
                         is_queued=True,
                         iswait_forfinish=True):
        return self.__dobotlink.Magician.SetPTPWithLCmd(
            portName=self.__port_name,
            ptpMode=ptp_mode,
            x=x,
            y=y,
            z=z,
            r=r,
            isQueued=is_queued,
            isWaitForFinish=iswait_forfinish)

    def set_ptpjoint_param(self, velocity, acceleration, is_queued=False):
        return self.__dobotlink.Magician.SetPTPJointParams(
            portName=self.__port_name,
            velocity=velocity,
            acceleration=acceleration,
            isQueued=is_queued)

    def get_ptpjoint_param(self):
        return self.__dobotlink.Magician.GetPTPJointParams(
            portName=self.__port_name)

    def set_ptpcoordinate_params(self,
                                 xyz_velocity,
                                 r_velocity,
                                 xyz_acceleration,
                                 r_acceleration,
                                 is_queued=False):
        return self.__dobotlink.Magician.SetPTPCoordinateParams(
            portName=self.__port_name,
            xyzVelocity=xyz_velocity,
            rVelocity=r_velocity,
            xyzAcceleration=xyz_acceleration,
            rAcceleration=r_acceleration,
            isQueued=is_queued)

    def get_ptpcoordinate_params(self):
        return self.__dobotlink.Magician.GetPTPCoordinateParams(
            portName=self.__port_name)

    def set_ptpjump_params(self, z_limit, jump_height, is_queued=False):
        return self.__dobotlink.Magician.SetPTPJumpParams(
            portName=self.__port_name,
            zLimit=z_limit,
            jumpHeight=jump_height,
            isQueued=is_queued)

    def get_ptpjump_params(self):
        return self.__dobotlink.Magician.GetPTPJumpParams(
            portName=self.__port_name)

    def set_ptpcommon_params(self,
                             velocity_ratio,
                             acceleration_ratio,
                             is_queued=False):
        return self.__dobotlink.Magician.SetPTPCommonParams(
            portName=self.__port_name,
            velocityRatio=velocity_ratio,
            accelerationRatio=acceleration_ratio,
            isQueued=is_queued)

    def get_ptpcommon_params(self):
        return self.__dobotlink.Magician.GetPTPCommonParams(
            portName=self.__port_name)

    def set_ptpl_params(self, velocity, acceleration, is_queued=False):
        return self.__dobotlink.Magician.SetPTPLParams(
            portName=self.__port_name,
            velocity=velocity,
            acceleration=acceleration,
            isQueued=is_queued)

    def get_ptpl_params(self):
        return self.__dobotlink.Magician.GetPTPLParams(
            portName=self.__port_name)

    def set_ptpjump2_params(self,
                            z_limit,
                            start_jump_height,
                            end_jump_height,
                            is_queued=False):
        return self.__dobotlink.Magician.SetPTPJump2Params(
            portName=self.__port_name,
            zLimit=z_limit,
            startJumpHeight=start_jump_height,
            endJumpHeight=end_jump_height,
            isQueued=is_queued)

    def get_ptpjump2_params(self):
        return self.__dobotlink.Magician.GetPTPJump2Params(
            portName=self.__port_name)

    def set_iomultiplexing(self, port: int, multiplex: int, is_queued=False):
        return self.__dobotlink.Magician.SetIOMultiplexing(
            portName=self.__port_name,
            port=port,
            multiplex=multiplex,
            isQueued=is_queued)

    def get_iomultiplexing(self, port: int):
        return self.__dobotlink.Magician.GetIOMultiplexing(
            portName=self.__port_name, port=port)

    def set_iodo(self, port, level, is_queued=False):
        return self.__dobotlink.Magician.SetIODO(portName=self.__port_name,
                                                 port=port,
                                                 level=level,
                                                 isQueued=is_queued)

    def get_iodo(self, port):
        return self.__dobotlink.Magician.GetIODO(portName=self.__port_name,
                                                 port=port)

    def set_iopwm(self, port, frequency, duty_cycle, is_queued=False):
        return self.__dobotlink.Magician.SetIOPWM(portName=self.__port_name,
                                                  port=port,
                                                  frequency=frequency,
                                                  dutyCycle=duty_cycle,
                                                  isQueued=is_queued)

    def get_iopwm(self, port):
        return self.__dobotlink.Magician.GetIOPWM(portName=self.__port_name,
                                                  port=port)

    def get_iodi(self, port):
        return self.__dobotlink.Magician.GetIODI(portName=self.__port_name,
                                                 port=port)

    def get_ioadc(self, port):
        return self.__dobotlink.Magician.GetIOADC(portName=self.__port_name,
                                                  port=port)

    def set_emotor(self, index, enable, speed, is_queued=False):
        return self.__dobotlink.Magician.SetEMotor(portName=self.__port_name,
                                                   index=index,
                                                   enable=enable,
                                                   speed=speed,
                                                   isQueued=is_queued)

    def set_emotors(self, index, enable, speed, distance, is_queued=False):
        return self.__dobotlink.Magician.SetEMotorS(portName=self.__port_name,
                                                    index=index,
                                                    enable=enable,
                                                    speed=speed,
                                                    distance=distance,
                                                    isQueued=is_queued)

    def set_color_sensor(self, port, enable, version, is_queued=False):
        return self.__dobotlink.Magician.SetColorSensor(
            portName=self.__port_name,
            port=port,
            enable=enable,
            version=version,
            isQueued=is_queued)

    def get_color_sensor(self):
        return self.__dobotlink.Magician.GetColorSensor(
            portName=self.__port_name)

    def set_infrared_sensor(self, port, enable, version, is_queued=False):
        return self.__dobotlink.Magician.SetInfraredSensor(
            portName=self.__port_name,
            port=port,
            enable=enable,
            version=version,
            isQueued=is_queued)

    def get_infrared_sensor(self, port):
        return self.__dobotlink.Magician.GetInfraredSensor(
            portName=self.__port_name, port=port)

    def set_loststep_value(self, value):
        return self.__dobotlink.Magician.SetLostStepValue(
            portName=self.__port_name, value=value)

    def set_loststep_cmd(self, is_queued=False):
        return self.__dobotlink.Magician.SetLostStepCmd(
            portName=self.__port_name, isQueued=is_queued)
# 3.12 连续运动轨迹

    def set_cpparams(self,
                     target_acc,
                     junction_vel,
                     isreal_timetrack,
                     acc=None,
                     period=None,
                     is_queued=False):
        return self.__dobotlink.Magician.SetCPParams(
            portName=self.__port_name,
            targetAcc=target_acc,
            junctionVel=junction_vel,
            isRealTimeTrack=isreal_timetrack,
            acc=acc,
            period=period,
            isQueued=is_queued)

    def get_cpparams(self):
        return self.__dobotlink.Magician.GetCPParams(portName=self.__port_name)

    def set_cpcmd(self, cp_mode, x, y, z, power, is_queued=False):
        return self.__dobotlink.Magician.SetCPCmd(portName=self.__port_name,
                                                  cpMode=cp_mode,
                                                  x=x,
                                                  y=y,
                                                  z=z,
                                                  power=power,
                                                  isQueued=is_queued)

    def set_cplecmd(self, cp_mode, x, y, z, power, is_queued=False):
        return self.__dobotlink.Magician.SetCPLECmd(portName=self.__port_name,
                                                    cpMode=cp_mode,
                                                    x=x,
                                                    y=y,
                                                    z=z,
                                                    power=power,
                                                    isQueued=is_queued)


# 3.13 圆弧插补功能

    def set_arcparams(self,
                      xyz_velocity,
                      r_velocity,
                      xyz_acceleration,
                      r_acceleration,
                      is_queued=False):
        return self.__dobotlink.Magician.SetARCParams(
            portName=self.__port_name,
            xyzVelocity=xyz_velocity,
            rVelocity=r_velocity,
            xyzAcceleration=xyz_acceleration,
            rAcceleration=r_acceleration,
            isQueued=is_queued)

    def get_arcparams(self):
        return self.__dobotlink.Magician.GetARCParams(
            portName=self.__port_name)

    def set_arccmd(self, cir_point, to_point, is_queued=False):
        return self.__dobotlink.Magician.SetARCCmd(portName=self.__port_name,
                                                   cirPoint=cir_point,
                                                   toPoint=to_point,
                                                   isQueued=is_queued)

    def set_anglesensorstatic_error(self, rear_armangle_error,
                                    front_armangle_error):
        return self.__dobotlink.Magician.SetAngleSensorStaticError(
            portName=self.__port_name,
            rearArmAngleError=rear_armangle_error,
            frontArmAngleError=front_armangle_error)

    def get_anglesensorstatic_error(self):
        return self.__dobotlink.Magician.GetAngleSensorStaticError(
            portName=self.__port_name)



import websockets
import asyncio
import json
from .NetworkError import NetworkError
from .Utils import loggers
from typing import Any, Callable


LOGGER_NAME = "RPCClient"


class RPCClient(object):
    def __init__(self,
                 ip: str = "127.0.0.1",
                 port: int = 9090,
                 cb_connected: Callable[[], None] = None,
                 cb_disconnected: Callable[[], None] = None,
                 loop: asyncio.BaseEventLoop = None):
        self.__cb_connected = cb_connected
        self.__cb_disconnected = cb_disconnected
        self.__loop = loop if loop else asyncio.get_event_loop()
        self.__id = 0
        self.__exchange_map = {}
        self.__recv_task_timer = None
        self.__ws: websockets.server.WebSocketServer = None
        self.__client_name = "DobotRPC-WSClient"
        self.__ip = ip
        self.__port = port

        self.__loop.create_task(self.__connect_recv_worker())

    def __del__(self):
        if self.__recv_task_timer:
            self.__recv_task_timer.stop()

        if self.__ws:
            self.__ws.close()
            self.__ws.wait_closed()

    @property
    def address(self):
        return f"{self.__ip}:{self.__port}"

    def __pack_rpc(self, rpc_id: int, rpc_playload: Any,
                   method: str = None) -> str:
        if rpc_id is None:
            data = {"jsonrpc": "2.0"}
        else:
            data = {"id": rpc_id, "jsonrpc": "2.0"}

        if method:
            data["method"] = method
            data["params"] = rpc_playload
        else:
            if isinstance(rpc_playload, Exception):
                # TODO: 所有异常捕捉并处理
                error_str = str(rpc_playload)
                if "code" in error_str:
                    rpc_playload = json.loads(error_str)
                else:
                    rpc_playload = {
                        "code": -32000,
                        "message": f"VM >> {error_str}"
                    }
                loggers.get(LOGGER_NAME).error(rpc_playload, exc_info=True)
                data["error"] = rpc_playload
            else:
                data["result"] = rpc_playload

        rpc_packet = json.dumps(data)
        return rpc_packet

    def __unpack_rpc(self, message: str) -> (int, str, str, str, Exception):
        try:
            data = json.loads(message)
        except Exception as e:
            raise NetworkError.InvaildJsonMsg(e)

        rpc_id = data.get("id", None)
        if type(rpc_id) is not int:
            raise NetworkError.RpcIdInvaild()

        rpc_verison = data.get("jsonrpc", None)
        if rpc_verison != "2.0":
            raise NetworkError.RpcVersionInvaild()

        method = data.get("method", None)
        params = data.get("params", None)
        result = data.get("result", None)
        error = data.get("error", None)
        if error:
            error = Exception(error)

        return rpc_id, method, params, result, error

    async def __connect_recv_worker(self):
        while True:
            if self.is_connected:
                try:
                    # 阻塞获取，不担心性能问题
                    ws_data = await self.__ws.recv()
                    loggers.get(LOGGER_NAME).debug(
                        f"rpc client <<<< {ws_data}")
                except Exception as e:
                    loggers.get(LOGGER_NAME).error(e, exc_info=True)
                    if self.__cb_disconnected:
                        result = self.__cb_disconnected()
                        if isinstance(result, asyncio.Future):
                            await result

                try:
                    rpc_id, _, _, result, error = self.__unpack_rpc(ws_data)
                    exchange = self.__exchange_map.get(rpc_id, None)
                    if exchange:
                        cb_feedback = exchange["feedback"]
                        if cb_feedback:
                            await cb_feedback(error, result)
                        else:
                            exchange["result"] = result
                            exchange["error"] = error
                            exchange["event"].set()
                    else:
                        loggers.get(LOGGER_NAME).warning(
                            f"Invalid request:{ws_data}")
                except Exception as e:
                    loggers.get(LOGGER_NAME).error(e, exc_info=True)

            else:
                try:
                    self.__ws = await websockets.connect(
                        f"ws://{self.__ip}:{self.__port}")
                    loggers.get(LOGGER_NAME).info("Have connected dobotlink")
                    if self.__cb_connected:
                        result = self.__cb_connected()
                        if isinstance(result, asyncio.Future):
                            await result
                except Exception as e:
                    loggers.get(LOGGER_NAME).warning(
                        f"Can not connect dobotlink: {e}")
                    await asyncio.sleep(0.5)

    async def send(self,
                   method: str,
                   params: dict = {}) -> Any:
        if not self.is_connected:
            raise NetworkError.CommNotConnectError(
                "Had not connected Dobotlink!")

        cb_feedback = params.pop("cb_feedback", None)

        self.__id += 1
        comm_id = self.__id

        payload = {
            "method": method,
            "jsonrpc": "2.0",
            "params": {} if params is None else params,
            "id": comm_id
        }

        loggers.get(LOGGER_NAME).debug(f"rpc client >>>> {payload}")
        await self.__ws.send(json.dumps(payload))

        if cb_feedback:
            self.__exchange_map[comm_id] = {
                "feedback": cb_feedback,
                "event": None,
                "result": None,
                "error": None
            }
            return None
        else:
            event = asyncio.Event()
            self.__exchange_map[comm_id] = {
                "feedback": None,
                "event": event,
                "result": None,
                "error": None
            }
            await event.wait()

            exchange = self.__exchange_map.pop(comm_id, None)
            if exchange is None:
                raise Exception()

            if exchange["error"] is not None:
                raise NetworkError.NetworkError(exchange["error"])

            return exchange["result"]

    @property
    def is_connected(self) -> bool:
        return self.__ws is not None and self.__ws.open

    async def wait_for_connected(self):
        while not self.is_connected:
            await asyncio.sleep(0.01)

    async def wait_for_disconnected(self):
        while self.is_connected:
            await asyncio.sleep(0.01)

    async def notify(self, method: str, data: Any) -> Any:
        if self.__ws_client is None:
            raise Exception("Had not connected!")

        rpc_packet = self.__pack_rpc(None, data, method)
        loggers.get(LOGGER_NAME).debug(f"rpc client >>>> {rpc_packet}")
        await self.__ws.send(json.dumps(rpc_packet))



from . import RPCClient
from typing import Any
import asyncio


class DobotlinkAdapter(object):
    class __ModuleAdapter(object):
        def __init__(self, rpc_client: RPCClient, is_sync: bool = False):
            self.__module_name = ""
            self.__rpc_client = rpc_client
            self.__is_sync = is_sync
            self.__port_name = None

        def set_port_name(self, port_name: str) -> None:
            self.__port_name = port_name

        def set_name(self, module_name: str) -> None:
            self.__module_name = module_name

        def __getattr__(self, func_name: str) -> Any:
            if self.__is_sync:

                def send_warpper(**params) -> Any:
                    method = f"dobotlink.{self.__module_name}.{func_name}"
                    if self.__port_name is not None:
                        params["portName"] = self.__port_name
                    fut = self.__rpc_client.send(method, params)
                    loop = asyncio.get_event_loop()
                    return loop.run_until_complete(fut)
            else:

                async def send_warpper(**params) -> Any:
                    method = f"dobotlink.{self.__module_name}.{func_name}"
                    if self.__port_name is not None:
                        params["portName"] = self.__port_name
                    return await self.__rpc_client.send(method, params)

            return send_warpper

    def __init__(self, rpc_client: RPCClient, is_sync: bool = False):
        self.__rpc_client = rpc_client
        self.__module = self.__ModuleAdapter(rpc_client, is_sync)

        loop = asyncio.get_event_loop()
        loop.run_until_complete(rpc_client.wait_for_connected())

    @property
    def is_connected(self):
        return self.__rpc_client and self.__rpc_client.is_connected

    def __getattr__(self, module_name):
        self.__module.set_name(module_name)
        return self.__module



from DobotRPC import DobotlinkAdapter, RPCClient


class LiteApi(object):
    def __init__(self):
        self.__port_name: str
        self.__dobotlink = DobotlinkAdapter(RPCClient(), is_sync=True)

    def set_portname(self, port_name):
        self.__port_name = port_name

    def search_dobot(self):
        return self.__dobotlink.MagicianLite.SearchDobot()

    def connect_dobot(self, queue_start=True):
        return self.__dobotlink.MagicianLite.ConnectDobot(
            portName=self.__port_name, queueStart=queue_start)

    def disconnect_dobot(self, queue_stop=True, queue_clear=True):
        return self.__dobotlink.MagicianLite.DisconnectDobot(
            portName=self.__port_name,
            queueStop=queue_stop,
            queueClear=queue_clear)

    def get_devicesn(self):
        return self.__dobotlink.MagicianLite.GetDeviceSN(
            portName=self.__port_name)

    def set_devicename(self, device_name):
        return self.__dobotlink.MagicianLite.SetDeviceName(
            portName=self.__port_name, deviceName=device_name)

    def get_devicename(self):
        return self.__dobotlink.MagicianLite.GetDeviceName(
            portName=self.__port_name)

    def get_deviceversion(self):
        return self.__dobotlink.MagicianLite.GetDeviceVersion(
            portName=self.__port_name)

    def get_devicetime(self):
        return self.__dobotlink.MagicianLite.GetDeviceTime(
            portName=self.__port_name)

    def get_deviceid(self):
        return self.__dobotlink.MagicianLite.GetDeviceID(
            portName=self.__port_name)

    def get_productname(self):
        return self.__dobotlink.MagicianLite.GetProductName(
            portName=self.__port_name)

    def queuedcmd_start(self):
        return self.__dobotlink.MagicianLite.QueuedCmdStart(
            portName=self.__port_name)

    def queuedcmd_stop(self, force_stop=False):
        return self.__dobotlink.MagicianLite.QueuedCmdStop(
            portName=self.__port_name, forceStop=force_stop)

    def queuedcmd_clear(self):
        return self.__dobotlink.MagicianLite.QueuedCmdClear(
            portName=self.__port_name)

    def queuedcmd_startdownload(self, total_loop, lineper_loop):
        return self.__dobotlink.MagicianLite.QueuedCmdStartDownload(
            portName=self.__port_name,
            totalLoop=total_loop,
            linePerLoop=lineper_loop)

    def queuedcmd_stopdownload(self):
        return self.__dobotlink.MagicianLite.QueuedCmdStopDownload(
            portName=self.__port_name)

    def get_queuedcmd_currentindex(self):
        return self.__dobotlink.MagicianLite.GetQueuedCmdCurrentIndex(
            portName=self.__port_name)

    def get_queuedcmd_leftspace(self):
        return self.__dobotlink.MagicianLite.GetQueuedCmdLeftSpace(
            portName=self.__port_name)

    def set_armspeed_ratio(self,
                           set_type: int,
                           set_value: int,
                           is_queued=False):
        return self.__dobotlink.MagicianLite.SetArmSpeedRatio(
            portName=self.__port_name,
            type=set_type,
            value=set_value,
            isQueued=is_queued)

    def get_armspeed_ratio(self, get_type: int):
        return self.__dobotlink.MagicianLite.GetArmSpeedRatio(
            portName=self.__port_name, type=get_type)

    def get_pose(self):
        return self.__dobotlink.MagicianLite.GetPose(portName=self.__port_name)

    def reset_pose(self,
                   manual_enable,
                   rear_armangle=None,
                   front_armangle=None):
        return self.__dobotlink.MagicianLite.ResetPose(
            portName=self.__port_name,
            manualEnable=manual_enable,
            rearArmAngle=rear_armangle,
            frontArmAngle=front_armangle)

    def check_poselimit(self, is_joint, x, y, z, r):
        return self.__dobotlink.MagicianLite.CheckPoseLimit(
            portName=self.__port_name, isJoint=is_joint, x=x, y=y, z=z, r=r)

    def get_alarms_state(self):
        return self.__dobotlink.MagicianLite.GetAlarmsState(
            portName=self.__port_name)

    def clear_allalarms_state(self):
        return self.__dobotlink.MagicianLite.ClearAllAlarmsState(
            portName=self.__port_name)

    def set_homeparams(self,
                       x: float,
                       y: float,
                       z: float,
                       r: float,
                       is_queued=False):
        return self.__dobotlink.MagicianLite.SetHOMEParams(
            portName=self.__port_name, x=x, y=y, z=z, r=r, isQueued=is_queued)

    def get_homeparams(self):
        return self.__dobotlink.MagicianLite.GetHOMEParams(
            portName=self.__port_name)

    def set_homecmd(self,
                    is_queued=True,
                    iswait_forfinish=True,
                    time_out=25000):
        return self.__dobotlink.MagicianLite.SetHOMECmd(
            portName=self.__port_name,
            isQueued=is_queued,
            isWaitForFinish=iswait_forfinish,
            timeout=time_out)

    def set_endeffector_params(self,
                               x_offset: float,
                               y_offset: float,
                               z_offset: float,
                               is_queued=False):
        return self.__dobotlink.MagicianLite.SetEndEffectorParams(
            portName=self.__port_name,
            xOffset=x_offset,
            yOffset=y_offset,
            zOffset=z_offset,
            isQueued=is_queued)

    def get_endeffector_params(self):
        return self.__dobotlink.MagicianLite.GetEndEffectorParams(
            portName=self.__port_name)

    def set_endeffector_type(self, set_type: int, is_queued=False):
        return self.__dobotlink.MagicianLite.SetEndEffectorType(
            portName=self.__port_name, type=set_type, isQueued=is_queued)

    def get_endeffector_type(self):
        return self.__dobotlink.MagicianLite.GetEndEffectorType(
            portName=self.__port_name)

    def set_endeffector_suctioncup(self,
                                   enable: bool,
                                   on: bool,
                                   is_queued=False):
        return self.__dobotlink.MagicianLite.SetEndEffectorSuctionCup(
            portName=self.__port_name,
            enable=enable,
            on=on,
            isQueued=is_queued)

    def get_endeffector_suctioncup(self):
        return self.__dobotlink.MagicianLite.GetEndEffectorSuctionCup(
            portName=self.__port_name)

    def set_endeffector_gripper(self, enable: bool, on: bool, is_queued=False):
        return self.__dobotlink.MagicianLite.SetEndEffectorGripper(
            portName=self.__port_name,
            enable=enable,
            on=on,
            isQueued=is_queued)

    def get_endeffector_gripper(self):
        return self.__dobotlink.MagicianLite.GetEndEffectorGripper(
            portName=self.__port_name)

    def set_jogjoint_params(self, velocity, acceleration, is_queued=False):
        return self.__dobotlink.MagicianLite.SetJOGJointParams(
            portName=self.__port_name,
            velocity=velocity,
            acceleration=acceleration,
            isQueued=is_queued)

    def get_jogjoint_params(self):
        return self.__dobotlink.MagicianLite.GetJOGJointParams(
            portName=self.__port_name)

    def set_jogcoordinate_params(self,
                                 velocity,
                                 acceleration,
                                 is_queued=False):
        return self.__dobotlink.MagicianLite.SetJOGCoordinateParams(
            portName=self.__port_name,
            velocity=velocity,
            acceleration=acceleration,
            isQueued=is_queued)

    def get_jogcoordinate_params(self):
        return self.__dobotlink.MagicianLite.GetJOGCoordinateParams(
            portName=self.__port_name)

    def set_jogcommon_params(self,
                             velocity_ratio,
                             acceleration_ratio,
                             is_queued=False):
        return self.__dobotlink.MagicianLite.SetJOGCommonParams(
            portName=self.__port_name,
            velocityRatio=velocity_ratio,
            accelerationRatio=acceleration_ratio,
            isQueued=is_queued)

    def get_jogcommon_params(self):
        return self.__dobotlink.MagicianLite.GetJOGCommonParams(
            portName=self.__port_name)

    def set_jogcmd(self, is_joint, cmd, is_queued):
        return self.__dobotlink.MagicianLite.SetJOGCmd(
            portName=self.__port_name,
            isJoint=is_joint,
            cmd=cmd,
            isQueued=is_queued)

    def set_ptpcmd(self,
                   ptp_mode,
                   x,
                   y,
                   z,
                   r,
                   is_queued=True,
                   iswait_forfinish=True):
        return self.__dobotlink.MagicianLite.SetPTPCmd(
            portName=self.__port_name,
            ptpMode=ptp_mode,
            x=x,
            y=y,
            z=z,
            r=r,
            isQueued=is_queued,
            isWaitForFinish=iswait_forfinish)

    def set_rcmd(self,
                 r: int,
                 is_queued=True,
                 iswait_forfinish=True,
                 time_out=5000):
        return self.__dobotlink.MagicianLite.SetRCmd(
            portName=self.__port_name,
            r=r,
            isQueued=is_queued,
            isWaitForFinish=iswait_forfinish,
            timeout=time_out)

    def set_ptpjoint_param(self, velocity, acceleration, is_queued=False):
        return self.__dobotlink.MagicianLite.SetPTPJointParams(
            portName=self.__port_name,
            velocity=velocity,
            acceleration=acceleration,
            isQueued=is_queued)

    def get_ptpjoint_param(self):
        return self.__dobotlink.MagicianLite.GetPTPJointParams(
            portName=self.__port_name)

    def set_ptpcoordinate_params(self,
                                 xyz_velocity,
                                 r_velocity,
                                 xyz_acceleration,
                                 r_acceleration,
                                 is_queued=False):
        return self.__dobotlink.MagicianLite.SetPTPCoordinateParams(
            portName=self.__port_name,
            xyzVelocity=xyz_velocity,
            rVelocity=r_velocity,
            xyzAcceleration=xyz_acceleration,
            rAcceleration=r_acceleration,
            isQueued=is_queued)

    def get_ptpcoordinate_params(self):
        return self.__dobotlink.MagicianLite.GetPTPCoordinateParams(
            portName=self.__port_name)

    def set_ptpjump_params(self, z_limit, jump_height, is_queued=False):
        return self.__dobotlink.MagicianLite.SetPTPJumpParams(
            portName=self.__port_name,
            zLimit=z_limit,
            jumpHeight=jump_height,
            isQueued=is_queued)

    def get_ptpjump_params(self):
        return self.__dobotlink.MagicianLite.GetPTPJumpParams(
            portName=self.__port_name)

    def set_ptpcommon_params(self,
                             velocity_ratio,
                             acceleration_ratio,
                             is_queued=False):
        return self.__dobotlink.MagicianLite.SetPTPCommonParams(
            portName=self.__port_name,
            velocityRatio=velocity_ratio,
            accelerationRatio=acceleration_ratio,
            isQueued=is_queued)

    def get_ptpcommon_params(self):
        return self.__dobotlink.MagicianLite.GetPTPCommonParams(
            portName=self.__port_name)

    def set_loststep_value(self, value):
        return self.__dobotlink.MagicianLite.SetLostStepValue(
            portName=self.__port_name, value=value)

    def set_loststep_cmd(self, is_queued=False):
        return self.__dobotlink.MagicianLite.SetLostStepCmd(
            portName=self.__port_name, isQueued=is_queued)

    def set_collision_check(self, enable, thre_shold):
        return self.__dobotlink.MagicianLite.SetCollisionCheck(
            portName=self.__port_name, enable=enable, threshold=thre_shold)

    def get_collision_check(self, is_queued=False):
        return self.__dobotlink.MagicianLite.GetCollisionCheck(
            portName=self.__port_name)

    def set_cpparams(self,
                     target_acc,
                     junction_vel,
                     isreal_timetrack,
                     acc=None,
                     period=None,
                     is_queued=False):
        return self.__dobotlink.MagicianLite.SetCPParams(
            portName=self.__port_name,
            targetAcc=target_acc,
            junctionVel=junction_vel,
            isRealTimeTrack=isreal_timetrack,
            acc=acc,
            period=period,
            isQueued=is_queued)

    def get_cpparams(self):
        return self.__dobotlink.MagicianLite.GetCPParams(
            portName=self.__port_name)

    def set_cpcmd(self, cp_mode, x, y, z, power, is_queued=False):
        return self.__dobotlink.MagicianLite.SetCPCmd(
            portName=self.__port_name,
            cpMode=cp_mode,
            x=x,
            y=y,
            z=z,
            power=power,
            isQueued=is_queued)

    def set_cplecmd(self, cp_mode, x, y, z, power, is_queued=False):
        return self.__dobotlink.MagicianLite.SetCPLECmd(
            portName=self.__port_name,
            cpMode=cp_mode,
            x=x,
            y=y,
            z=z,
            power=power,
            isQueued=is_queued)

    def set_arcparams(self,
                      xyz_velocity,
                      r_velocity,
                      xyz_acceleration,
                      r_acceleration,
                      is_queued=False):
        return self.__dobotlink.MagicianLite.SetARCParams(
            portName=self.__port_name,
            xyzVelocity=xyz_velocity,
            rVelocity=r_velocity,
            xyzAcceleration=xyz_acceleration,
            rAcceleration=r_acceleration,
            isQueued=is_queued)

    def get_arcparams(self):
        return self.__dobotlink.MagicianLite.GetARCParams(
            portName=self.__port_name)

    def set_arccmd(self, cir_point, to_point, is_queued=False):
        return self.__dobotlink.MagicianLite.SetARCCmd(
            portName=self.__port_name,
            cirPoint=cir_point,
            toPoint=to_point,
            isQueued=is_queued)
